@@ -23,11 +23,11 @@
 #include <config.h>
 #include <string.h>
 
-#include "gtkalias.h"
 #include "gtkclipboard.h"
 #include "gtkinvisible.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
+#include "gtkalias.h"
 
 #ifdef GDK_WINDOWING_X11
 #include "x11/gdkx.h"
@@ -260,8 +260,8 @@ clipboard_display_closed (GdkDisplay   *display,
  *
  * (Passing #GDK_NONE is the same as using <literal>gdk_atom_intern
  * ("CLIPBOARD", FALSE)</literal>. See <ulink
- * url="http://www.freedesktop.org/standards/clipboards-spec/clipboards.txt">
- * http://www.freedesktop.org/standards/clipboards-spec/clipboards.txt</ulink>
+ * url="http://www.freedesktop.org/Standards/clipboards-spec">
+ * http://www.freedesktop.org/Standards/clipboards-spec</ulink>
  * for a detailed discussion of the "CLIPBOARD" vs. "PRIMARY"
  * selections under the X window system. On Win32 the
  * #GDK_SELECTION_PRIMARY clipboard is essentially ignored.)
@@ -678,7 +678,7 @@ clipboard_unset (GtkClipboard *clipboard)
    */
   if (old_have_owner &&
       old_n_storable_targets != -1)
-    g_object_unref (clipboard->user_data);
+    g_object_unref (old_data);
 }
 
 /**
@@ -765,6 +765,8 @@ gtk_clipboard_set_text (GtkClipboard *clipboard,
 			       g_strndup (text, len));
   gtk_clipboard_set_can_store (clipboard, NULL, 0);
 
+  for (i = 0; i < n_targets; i++)
+    g_free (targets[i].target);
   g_free (targets);
   gtk_target_list_unref (list);
 }
@@ -826,6 +828,8 @@ gtk_clipboard_set_image (GtkClipboard *clipboard,
 			       g_object_ref (pixbuf));
   gtk_clipboard_set_can_store (clipboard, NULL, 0);
 
+  for (i = 0; i < n_targets; i++)
+    g_free (targets[i].target);
   g_free (targets);
   gtk_target_list_unref (list);
 }
@@ -1000,6 +1004,7 @@ request_image_received_func (GtkClipboard     *clipboard,
     {
       /* If we asked for image/png and didn't get it, try image/jpeg;
        * if we asked for image/jpeg and didn't get it, try image/gif;
+       * if we asked for image/gif and didn't get it, try image/bmp;
        * If we asked for anything else and didn't get it, give up.
        */
       if (selection_data->target == gdk_atom_intern ("image/png", FALSE))
@@ -1013,7 +1018,14 @@ request_image_received_func (GtkClipboard     *clipboard,
 	{
 	  gtk_clipboard_request_contents (clipboard,
 					  gdk_atom_intern ("image/gif", FALSE), 
-					  request_text_received_func, info);
+					  request_image_received_func, info);
+	  return;
+	}
+      else if (selection_data->target == gdk_atom_intern ("image/gif", FALSE))
+	{
+	  gtk_clipboard_request_contents (clipboard,
+					  gdk_atom_intern ("image/bmp", FALSE), 
+					  request_image_received_func, info);
 	  return;
 	}
     }
@@ -1609,6 +1621,7 @@ gtk_clipboard_set_can_store (GtkClipboard         *clipboard,
   };
   
   g_return_if_fail (GTK_IS_CLIPBOARD (clipboard));
+  g_return_if_fail (n_targets >= 0);
 
   if (clipboard->selection != GDK_SELECTION_CLIPBOARD)
     return;
@@ -1731,3 +1744,6 @@ _gtk_clipboard_store_all (void)
   g_slist_free (displays);
   
 }
+
+#define __GTK_CLIPBOARD_C__
+#include "gtkaliasdef.c"
