@@ -25,9 +25,10 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkfixed.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 enum {
   CHILD_PROP_0,
@@ -124,7 +125,7 @@ gtk_fixed_class_init (GtkFixedClass *class)
                                                                 G_MININT,
                                                                 G_MAXINT,
                                                                 0,
-                                                                G_PARAM_READWRITE));
+                                                                GTK_PARAM_READWRITE));
 
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_Y,
@@ -134,7 +135,7 @@ gtk_fixed_class_init (GtkFixedClass *class)
                                                                 G_MININT,
                                                                 G_MAXINT,
                                                                 0,
-                                                                G_PARAM_READWRITE));
+                                                                GTK_PARAM_READWRITE));
 }
 
 static GType
@@ -187,7 +188,7 @@ gtk_fixed_put (GtkFixed       *fixed,
   GtkFixedChild *child_info;
 
   g_return_if_fail (GTK_IS_FIXED (fixed));
-  g_return_if_fail (GTK_IS_WIDGET (fixed));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
 
   child_info = g_new (GtkFixedChild, 1);
   child_info->widget = widget;
@@ -532,3 +533,6 @@ gtk_fixed_get_has_window (GtkFixed *fixed)
 
   return !GTK_WIDGET_NO_WINDOW (fixed);
 }
+
+#define __GTK_FIXED_C__
+#include "gtkaliasdef.c"
