@@ -52,6 +52,7 @@
 #include "gtktextchild.h"
 #include "gtktextbtree.h"
 #include "gtktextlayout.h"
+#include "gtkalias.h"
 
 #define CHECK_IN_BUFFER(anchor)                                                            \
   G_STMT_START {                                                                           \
@@ -538,4 +539,5 @@ gtk_text_anchored_child_set_layout (GtkWidget     *child,
   _gtk_anchored_child_set_layout (child, layout);
 }
 
-
+#define __GTK_TEXT_CHILD_C__
+#include "gtkaliasdef.c"
