@@ -24,11 +24,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gtklabel.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtktogglebutton.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 #define DEFAULT_LEFT_POS  4
 #define DEFAULT_TOP_POS   4
@@ -132,7 +135,7 @@ gtk_toggle_button_class_init (GtkToggleButtonClass *class)
 							 P_("Active"),
 							 P_("If the toggle button should be pressed in or not"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_INCONSISTENT,
@@ -140,15 +143,15 @@ gtk_toggle_button_class_init (GtkToggleButtonClass *class)
 							 P_("Inconsistent"),
 							 P_("If the toggle button is in an \"in between\" state"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_DRAW_INDICATOR,
-                                   g_param_spec_boolean ("draw_indicator",
+                                   g_param_spec_boolean ("draw-indicator",
 							 P_("Draw Indicator"),
 							 P_("If the toggle part of the button is displayed"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
 
   toggle_button_signals[TOGGLED] =
     g_signal_new ("toggled",
@@ -194,7 +197,10 @@ gtk_toggle_button_new_with_label (const gchar *label)
 GtkWidget*
 gtk_toggle_button_new_with_mnemonic (const gchar *label)
 {
-  return g_object_new (GTK_TYPE_TOGGLE_BUTTON, "label", label, "use_underline", TRUE, NULL);
+  return g_object_new (GTK_TYPE_TOGGLE_BUTTON, 
+		       "label", label, 
+		       "use-underline", TRUE, 
+		       NULL);
 }
 
 static void
@@ -260,7 +266,7 @@ gtk_toggle_button_get_property (GObject      *object,
  * You can call this function on a checkbutton or a radiobutton with
  * @draw_indicator = %FALSE to make the button look like a normal button
  *
- * This function only effects instances of classes like #GtkCheckButton
+ * This function only affects instances of classes like #GtkCheckButton
  * and #GtkRadioButton that derive from #GtkToggleButton,
  * not instances of #GtkToggleButton itself.
  */
@@ -284,7 +290,7 @@ gtk_toggle_button_set_mode (GtkToggleButton *toggle_button,
       if (GTK_WIDGET_VISIBLE (toggle_button))
 	gtk_widget_queue_resize (GTK_WIDGET (toggle_button));
 
-      g_object_notify (G_OBJECT (toggle_button), "draw_indicator");
+      g_object_notify (G_OBJECT (toggle_button), "draw-indicator");
     }
 }
 
@@ -495,3 +501,6 @@ gtk_toggle_button_update_state (GtkButton *button)
   _gtk_button_set_depressed (button, depressed); 
   gtk_widget_set_state (GTK_WIDGET (toggle_button), new_state);
 }
+
+#define __GTK_TOGGLE_BUTTON_C__
+#include "gtkaliasdef.c"
